@@ -1,18 +1,16 @@
 from django.db.utils import DatabaseError
 
 try:
-    from django.utils.six.moves import _thread as thread
+    import thread
 except ImportError:
-    from django.utils.six.moves import _dummy_thread as thread
+    import dummy_thread as thread
 from contextlib import contextmanager
 
 from django.conf import settings
 from django.db import DEFAULT_DB_ALIAS
 from django.db.backends import util
 from django.db.transaction import TransactionManagementError
-from django.utils.functional import cached_property
 from django.utils.importlib import import_module
-from django.utils import six
 from django.utils.timezone import is_aware
 
 
@@ -47,8 +45,6 @@ class BaseDatabaseWrapper(object):
     def __ne__(self, other):
         return not self == other
 
-    __hash__ = object.__hash__
-
     def _commit(self):
         if self.connection is not None:
             return self.connection.commit()
@@ -112,18 +108,16 @@ class BaseDatabaseWrapper(object):
         over to the surrounding block, as a commit will commit all changes, even
         those from outside. (Commits are on connection level.)
         """
+        self._leave_transaction_management(self.is_managed())
         if self.transaction_state:
             del self.transaction_state[-1]
         else:
-            raise TransactionManagementError(
-                "This code isn't under transaction management")
-        # We will pass the next status (after leaving the previous state
-        # behind) to subclass hook.
-        self._leave_transaction_management(self.is_managed())
+            raise TransactionManagementError("This code isn't under transaction "
+                "management")
         if self._dirty:
             self.rollback()
-            raise TransactionManagementError(
-                "Transaction managed block ended with pending COMMIT/ROLLBACK")
+            raise TransactionManagementError("Transaction managed block ended with "
+                "pending COMMIT/ROLLBACK")
         self._dirty = False
 
     def validate_thread_sharing(self):
@@ -181,8 +175,6 @@ class BaseDatabaseWrapper(object):
         """
         if self.transaction_state:
             return self.transaction_state[-1]
-        # Note that this setting isn't documented, and is only used here, and
-        # in enter_transaction_management()
         return settings.TRANSACTIONS_MANAGED
 
     def managed(self, flag=True):
@@ -319,7 +311,6 @@ class BaseDatabaseWrapper(object):
     def make_debug_cursor(self, cursor):
         return util.CursorDebugWrapper(cursor, self)
 
-
 class BaseDatabaseFeatures(object):
     allows_group_by_pk = False
     # True if django.db.backend.utils.typecast_timestamp is used on values
@@ -408,13 +399,12 @@ class BaseDatabaseFeatures(object):
     # in the SQL standard.
     supports_tablespaces = False
 
-    # Does the backend reset sequences between tests?
-    supports_sequence_reset = True
-
-    # Confirm support for introspected foreign keys
-    # Every database can do this reliably, except MySQL,
-    # which can't do it for MyISAM tables
-    can_introspect_foreign_keys = True
+    # Features that need to be confirmed at runtime
+    # Cache whether the confirmation has been performed.
+    _confirmed = False
+    supports_transactions = None
+    supports_stddev = None
+    can_introspect_foreign_keys = None
 
     # Support for the DISTINCT ON clause
     can_distinct_on_fields = False
@@ -422,40 +412,41 @@ class BaseDatabaseFeatures(object):
     def __init__(self, connection):
         self.connection = connection
 
-    @cached_property
-    def supports_transactions(self):
+    def confirm(self):
+        "Perform manual checks of any database features that might vary between installs"
+        self._confirmed = True
+        self.supports_transactions = self._supports_transactions()
+        self.supports_stddev = self._supports_stddev()
+        self.can_introspect_foreign_keys = self._can_introspect_foreign_keys()
+
+    def _supports_transactions(self):
         "Confirm support for transactions"
-        try:
-            # Make sure to run inside a managed transaction block,
-            # otherwise autocommit will cause the confimation to
-            # fail.
-            self.connection.enter_transaction_management()
-            self.connection.managed(True)
-            cursor = self.connection.cursor()
-            cursor.execute('CREATE TABLE ROLLBACK_TEST (X INT)')
-            self.connection._commit()
-            cursor.execute('INSERT INTO ROLLBACK_TEST (X) VALUES (8)')
-            self.connection._rollback()
-            cursor.execute('SELECT COUNT(X) FROM ROLLBACK_TEST')
-            count, = cursor.fetchone()
-            cursor.execute('DROP TABLE ROLLBACK_TEST')
-            self.connection._commit()
-            self.connection._dirty = False
-        finally:
-            self.connection.leave_transaction_management()
+        cursor = self.connection.cursor()
+        cursor.execute('CREATE TABLE ROLLBACK_TEST (X INT)')
+        self.connection._commit()
+        cursor.execute('INSERT INTO ROLLBACK_TEST (X) VALUES (8)')
+        self.connection._rollback()
+        cursor.execute('SELECT COUNT(X) FROM ROLLBACK_TEST')
+        count, = cursor.fetchone()
+        cursor.execute('DROP TABLE ROLLBACK_TEST')
+        self.connection._commit()
         return count == 0
 
-    @cached_property
-    def supports_stddev(self):
+    def _supports_stddev(self):
         "Confirm support for STDDEV and related stats functions"
         class StdDevPop(object):
             sql_function = 'STDDEV_POP'
 
         try:
             self.connection.ops.check_aggregate_support(StdDevPop())
-            return True
         except NotImplementedError:
-            return False
+            self.supports_stddev = False
+
+    def _can_introspect_foreign_keys(self):
+        "Confirm support for introspected foreign keys"
+        # Every database can do this reliably, except MySQL,
+        # which can't do it for MyISAM tables
+        return True
 
 
 class BaseDatabaseOperations(object):
@@ -487,16 +478,6 @@ class BaseDatabaseOperations(object):
         """
         return len(objs)
 
-    def cache_key_culling_sql(self):
-        """
-        Returns a SQL query that retrieves the first cache key greater than the
-        n smallest.
-
-        This is used by the 'db' cache backend to determine where to start
-        culling.
-        """
-        return "SELECT cache_key FROM %s ORDER BY cache_key LIMIT 1 OFFSET %%s"
-
     def date_extract_sql(self, lookup_type, field_name):
         """
         Given a lookup_type of 'year', 'month' or 'day', returns the SQL that
@@ -610,16 +591,16 @@ class BaseDatabaseOperations(object):
         exists for database backends to provide a better implementation
         according to their own quoting schemes.
         """
-        from django.utils.encoding import force_text
+        from django.utils.encoding import smart_unicode, force_unicode
 
         # Convert params to contain Unicode values.
-        to_unicode = lambda s: force_text(s, strings_only=True, errors='replace')
+        to_unicode = lambda s: force_unicode(s, strings_only=True, errors='replace')
         if isinstance(params, (list, tuple)):
             u_params = tuple([to_unicode(val) for val in params])
         else:
             u_params = dict([(to_unicode(k), to_unicode(v)) for k, v in params.items()])
 
-        return force_text(sql) % u_params
+        return smart_unicode(sql) % u_params
 
     def last_insert_id(self, cursor, table_name, pk_name):
         """
@@ -751,24 +732,11 @@ class BaseDatabaseOperations(object):
         the given database tables (without actually removing the tables
         themselves).
 
-        The returned value also includes SQL statements required to reset DB
-        sequences passed in :param sequences:.
-
         The `style` argument is a Style object as returned by either
         color_style() or no_style() in django.core.management.color.
         """
         raise NotImplementedError()
 
-    def sequence_reset_by_name_sql(self, style, sequences):
-        """
-        Returns a list of the SQL statements required to reset sequences
-        passed in :param sequences:.
-
-        The `style` argument is a Style object as returned by either
-        color_style() or no_style() in django.core.management.color.
-        """
-        return []
-
     def sequence_reset_sql(self, style, model_list):
         """
         Returns a list of the SQL statements required to reset sequences for
@@ -777,7 +745,7 @@ class BaseDatabaseOperations(object):
         The `style` argument is a Style object as returned by either
         color_style() or no_style() in django.core.management.color.
         """
-        return []  # No sequence reset required by default.
+        return [] # No sequence reset required by default.
 
     def start_transaction_sql(self):
         """
@@ -803,21 +771,13 @@ class BaseDatabaseOperations(object):
 
     def prep_for_like_query(self, x):
         """Prepares a value for use in a LIKE query."""
-        from django.utils.encoding import force_text
-        return force_text(x).replace("\\", "\\\\").replace("%", "\%").replace("_", "\_")
+        from django.utils.encoding import smart_unicode
+        return smart_unicode(x).replace("\\", "\\\\").replace("%", "\%").replace("_", "\_")
 
     # Same as prep_for_like_query(), but called for "iexact" matches, which
     # need not necessarily be implemented using "LIKE" in the backend.
     prep_for_iexact_query = prep_for_like_query
 
-    def validate_autopk_value(self, value):
-        """
-        Certain backends do not accept some values for "serial" fields
-        (for example zero in MySQL). This method will raise a ValueError
-        if the value is invalid, otherwise returns validated value.
-        """
-        return value
-
     def value_to_db_date(self, value):
         """
         Transform a date value to an object compatible with what is expected
@@ -825,7 +785,7 @@ class BaseDatabaseOperations(object):
         """
         if value is None:
             return None
-        return six.text_type(value)
+        return unicode(value)
 
     def value_to_db_datetime(self, value):
         """
@@ -834,7 +794,7 @@ class BaseDatabaseOperations(object):
         """
         if value is None:
             return None
-        return six.text_type(value)
+        return unicode(value)
 
     def value_to_db_time(self, value):
         """
@@ -845,7 +805,7 @@ class BaseDatabaseOperations(object):
             return None
         if is_aware(value):
             raise ValueError("Django does not support timezone-aware times.")
-        return six.text_type(value)
+        return unicode(value)
 
     def value_to_db_decimal(self, value, max_digits, decimal_places):
         """
@@ -881,21 +841,19 @@ class BaseDatabaseOperations(object):
         return self.year_lookup_bounds(value)
 
     def convert_values(self, value, field):
-        """
-        Coerce the value returned by the database backend into a consistent type
-        that is compatible with the field type.
+        """Coerce the value returned by the database backend into a consistent type that
+        is compatible with the field type.
         """
         internal_type = field.get_internal_type()
         if internal_type == 'DecimalField':
             return value
-        elif internal_type == 'FloatField':
-            return float(value)
-        elif (internal_type and (internal_type.endswith('IntegerField')
-                                 or internal_type == 'AutoField')):
+        elif internal_type and internal_type.endswith('IntegerField') or internal_type == 'AutoField':
             return int(value)
-        elif internal_type in ('DateField', 'DateTimeField', 'TimeField'):
+        elif internal_type in ('DateField', 'DateTimeField', 'TimeField', 'CharField'): #Vijay, vijay
             return value
-        return value
+        # No field, or the field isn't known to be a decimal or integer
+        # Default to a float
+        return float(value)
 
     def check_aggregate_support(self, aggregate_func):
         """Check that the backend supports the provided aggregate
@@ -916,7 +874,6 @@ class BaseDatabaseOperations(object):
         conn = ' %s ' % connector
         return conn.join(sub_expressions)
 
-
 class BaseDatabaseIntrospection(object):
     """
     This class encapsulates all backend-specific introspection utilities
@@ -941,23 +898,10 @@ class BaseDatabaseIntrospection(object):
         """
         return name
 
-    def table_names(self, cursor=None):
-        """
-        Returns a list of names of all tables that exist in the database.
-        The returned table list is sorted by Python's default sorting. We
-        do NOT use database's ORDER BY here to avoid subtle differences
-        in sorting order between databases.
-        """
-        if cursor is None:
-            cursor = self.connection.cursor()
-        return sorted(self.get_table_list(cursor))
-
-    def get_table_list(self, cursor):
-        """
-        Returns an unsorted list of names of all tables that exist in the
-        database.
-        """
-        raise NotImplementedError
+    def table_names(self):
+        "Returns a list of names of all tables that exist in the database."
+        cursor = self.connection.cursor()
+        return self.get_table_list(cursor)
 
     def django_table_names(self, only_existing=False):
         """
@@ -995,7 +939,7 @@ class BaseDatabaseIntrospection(object):
             for model in models.get_models(app):
                 if router.allow_syncdb(self.connection.alias, model):
                     all_models.append(model)
-        tables = list(map(self.table_name_converter, tables))
+        tables = map(self.table_name_converter, tables)
         return set([
             m for m in all_models
             if self.table_name_converter(m._meta.db_table) in tables
@@ -1012,14 +956,12 @@ class BaseDatabaseIntrospection(object):
             for model in models.get_models(app):
                 if not model._meta.managed:
                     continue
-                if model._meta.swapped:
-                    continue
                 if not router.allow_syncdb(self.connection.alias, model):
                     continue
                 for f in model._meta.local_fields:
                     if isinstance(f, models.AutoField):
                         sequence_list.append({'table': model._meta.db_table, 'column': f.column})
-                        break  # Only one AutoField is allowed per model, so don't bother continuing.
+                        break # Only one AutoField is allowed per model, so don't bother continuing.
 
                 for f in model._meta.local_many_to_many:
                     # If this is an m2m using an intermediate table,
@@ -1038,25 +980,10 @@ class BaseDatabaseIntrospection(object):
 
     def get_primary_key_column(self, cursor, table_name):
         """
-        Returns the name of the primary key column for the given table.
-        """
-        for column in six.iteritems(self.get_indexes(cursor, table_name)):
-            if column[1]['primary_key']:
-                return column[0]
-        return None
-
-    def get_indexes(self, cursor, table_name):
-        """
-        Returns a dictionary of indexed fieldname -> infodict for the given
-        table, where each infodict is in the format:
-            {'primary_key': boolean representing whether it's the primary key,
-             'unique': boolean representing whether it's a unique index}
-
-        Only single-column indexes are introspected.
+        Backends can override this to return the column name of the primary key for the given table.
         """
         raise NotImplementedError
 
-
 class BaseDatabaseClient(object):
     """
     This class encapsulates all backend-specific methods for opening a
@@ -1073,7 +1000,6 @@ class BaseDatabaseClient(object):
     def runshell(self):
         raise NotImplementedError()
 
-
 class BaseDatabaseValidation(object):
     """
     This class encapsualtes all backend-specific model validation.
